
/*
 ** Copyright (c) 2005
 **
 **
 */

#define VERSION "V0.01"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <stdint.h>


#include "vserver.h"
#include "switch.h"
#include "context.h"
#include "context_cmd.h"
#include "legacy.h"

static	char	err_flag = 0;

static	char *	cmd_name = NULL;
static	char *	set_arg = NULL;

static	int 	num_xid = 0;
static	int 	num_flag = 0;

#define LEGACY 1
#define RECENT 2

void new_context(int method)
{
	struct vcmd_new_s_context_v1 data = {0,0};
	int ctx = 0;

	switch (method) {
		case LEGACY:
			ctx = vserver(VCMD_new_s_context, -1, &data);
			break;
		case RECENT:
			ctx = vserver(VCMD_ctx_create, VX_DYNAMIC_ID, NULL);
			break;
	}
	printf("New ctx: %d\n", ctx);
}

int linear_fork(int num)
{
	int i;

	for (i = 0; num == -1 || i < num; i++)
		if (!fork())
			return 0;

	return 1;
}

int binary_fork(int max)
{
	int current = 0;

	if (fork())
		return 1;

	while(max == -1 || current++ < max)
		fork();

	return 0;
}

#define OPTIONS "+hb:l:oV"

int	main(int argc, char *argv[])
{
	extern int optind;
	extern char *optarg;
	char c, errflg = 0;
	int r = 0;
	char opt_vers = 0;
	char opt_binfork = 0;
	char opt_linfork = 0;
	int forks;
	int syscall_method = RECENT;

	cmd_name = argv[0];
	while ((c = getopt(argc, argv, OPTIONS)) != EOF) {
		switch (c) {
			case 'h':
				fprintf(stderr,
						"This is %s " VERSION "\n"
						"options are:\n"
						"-h         print this help message\n"
						"-V         verify interface version\n"
						"-b <depth> binary forking, <depth> levels (-1 = unlimited)\n"
						"-l <count> linear forking, <count> times (-1 = unlimited)\n"
						"-o         use legacy syscall interface\n"
						"--         end of options\n"
						,cmd_name);
				exit(0);
				break;
			case 'b':
				opt_binfork = 1;
				forks = atoi(optarg);
				break;
			case 'l':
				opt_linfork = 1;
				forks = atoi(optarg);
				break;
			case 'o':
				syscall_method = LEGACY;
				break;
			case 'V':
				opt_vers = 1;
				break;
			case '?':
			default:
				errflg++;
				break;
		}
	}
	if (errflg) {
		fprintf(stderr, 
				"Usage: %s -[" OPTIONS "] <path> ...\n"
				"%s -h for help.\n",
				cmd_name, cmd_name);
		exit(2);
	}

	if (opt_linfork && opt_binfork) {
		fprintf(stderr,
				"Can't do binary and linear fork at once\n");
		exit(2);
	}

	if (opt_vers) {
		r = vserver(VCMD_get_version, 0, NULL);
		if (r<0)
			perror("vc_get_version");
		else 
			printf("version: %04x:%04x\n", 
					(r>>16)&0xFFFF, r&0xFFFF);
	}

	signal(SIGCHLD, SIG_IGN);

	if (opt_linfork) {
		r = linear_fork(forks);
	}

	if (opt_binfork) {
		r = binary_fork(forks);
	}

	if (!r)
		new_context(syscall_method);

	exit((err_flag)?1:0);
}

